@@ -921,12 +921,28 @@ gkr_pk_object_index_has_value (GkrPkObject *object, const gchar *field)
 	if (object->storage) {
 		g_return_val_if_fail (GKR_IS_PK_STORAGE (object->storage), FALSE);
 		index = gkr_pk_storage_index (object->storage, object->location);
-		g_return_val_if_fail (index, FALSE);
 	} 
 	
 	return gkr_pk_index_has_value (index, object->digest, field);
 }
 
+gboolean
+gkr_pk_object_index_get_boolean (GkrPkObject *object, const gchar *field)
+{
+	GkrPkIndex *index = NULL;
+
+	g_return_val_if_fail (GKR_IS_PK_OBJECT (object), FALSE);
+	g_return_val_if_fail (object->digest, FALSE);
+	g_return_val_if_fail (field, FALSE);
+	
+	if (object->storage) {
+		g_return_val_if_fail (GKR_IS_PK_STORAGE (object->storage), FALSE);
+		index = gkr_pk_storage_index (object->storage, object->location);
+	} 
+	
+	return gkr_pk_index_get_boolean (index, object->digest, field, FALSE);
+}
+
 GQuark*
 gkr_pk_object_index_get_quarks (GkrPkObject *object, const gchar *field)
 {
@@ -937,9 +953,8 @@ gkr_pk_object_index_get_quarks (GkrPkObject *object, const gchar *field)
 	g_return_val_if_fail (field, NULL);
 	
 	if (object->storage) {
-		g_return_val_if_fail (GKR_IS_PK_STORAGE (object->storage), FALSE);
+		g_return_val_if_fail (GKR_IS_PK_STORAGE (object->storage), NULL);
 		index = gkr_pk_storage_index (object->storage, object->location);
-		g_return_val_if_fail (index, FALSE);
 	} 
 	
 	return gkr_pk_index_get_quarks (index, object->digest, field);
@@ -955,9 +970,8 @@ gkr_pk_object_index_get_string (GkrPkObject *object, const gchar *field)
 	g_return_val_if_fail (field, NULL);
 	
 	if (object->storage) {
-		g_return_val_if_fail (GKR_IS_PK_STORAGE (object->storage), FALSE);
+		g_return_val_if_fail (GKR_IS_PK_STORAGE (object->storage), NULL);
 		index = gkr_pk_storage_index (object->storage, object->location);
-		g_return_val_if_fail (index, FALSE);
 	} 
 	
 	return gkr_pk_index_get_string (index, object->digest, field);
@@ -974,9 +988,8 @@ gkr_pk_object_index_get_binary (GkrPkObject *object, const gchar *field,
 	g_return_val_if_fail (field, NULL);
 	
 	if (object->storage) {
-		g_return_val_if_fail (GKR_IS_PK_STORAGE (object->storage), FALSE);
+		g_return_val_if_fail (GKR_IS_PK_STORAGE (object->storage), NULL);
 		index = gkr_pk_storage_index (object->storage, object->location);
-		g_return_val_if_fail (index, FALSE);
 	} 
 	
 	return gkr_pk_index_get_binary (index, object->digest, field, n_data);
@@ -995,7 +1008,6 @@ gkr_pk_object_index_set_boolean (GkrPkObject *object, const gchar *field,
 	if (object->storage) {
 		g_return_if_fail (GKR_IS_PK_STORAGE (object->storage));
 		index = gkr_pk_storage_index (object->storage, object->location);
-		g_return_if_fail (index);
 	} 
 	
 	if (gkr_pk_index_set_boolean (index, object->digest, field, value))
@@ -1015,7 +1027,6 @@ gkr_pk_object_index_set_string (GkrPkObject *object, const gchar *field,
 	if (object->storage) {
 		g_return_if_fail (GKR_IS_PK_STORAGE (object->storage));
 		index = gkr_pk_storage_index (object->storage, object->location);
-		g_return_if_fail (index);
 	} 
 	
 	if (gkr_pk_index_set_string (index, object->digest, field, string))
@@ -1035,7 +1046,6 @@ gkr_pk_object_index_set_binary (GkrPkObject *object, const gchar *field,
 	if (object->storage) {
 		g_return_if_fail (GKR_IS_PK_STORAGE (object->storage));
 		index = gkr_pk_storage_index (object->storage, object->location);
-		g_return_if_fail (index);
 	}
 	
 	if (gkr_pk_index_set_binary (index, object->digest, field, data, n_data))
@@ -1054,7 +1064,6 @@ gkr_pk_object_index_clear (GkrPkObject *object, const gchar *field)
 	if (object->storage) {
 		g_return_if_fail (GKR_IS_PK_STORAGE (object->storage));
 		index = gkr_pk_storage_index (object->storage, object->location);
-		g_return_if_fail (index);
 	}
 	
 	if (gkr_pk_index_clear (index, object->digest, field))
