@@ -225,9 +225,9 @@ prepare_ask_store_secondary (GQuark type, const gchar *label)
 	 */
 
 	if (type == GKR_PKIX_PRIVATE_KEY)
-		return g_strdup_printf (_("The system wants to store to the private key '%s' on your disk. Please enter a password to lock it with."), label);
+		return g_strdup_printf (_("The system wants to store the private key '%s' on your disk. Please enter a password to lock it with."), label);
 	else 
-		return g_strdup_printf (_("The system wants to store to '%s' on your disk. Please enter a password to lock it with."), label);
+		return g_strdup_printf (_("The system wants to store '%s' on your disk. Please enter a password to lock it with."), label);
 }
 
 static void
@@ -287,15 +287,11 @@ static GkrPkIndex*
 gkr_pk_storage_internal_index (GkrPkStorage *storage, GQuark unused)
 {
  	GkrPkStoragePrivate *pv = GKR_PK_STORAGE_GET_PRIVATE (storage);
-	GQuark kloc;
 	
 	if (!pv->index) {
-		/* We default to a keyring stored on the computer */
-		kloc = gkr_location_from_child (GKR_LOCATION_VOLUME_LOCAL, 
-		                                "pk-storage.keyring");
-	
-		pv->index = gkr_pk_index_open (kloc, "pk-storage", NULL);
-		g_return_val_if_fail (pv->index, NULL);
+		pv->index = gkr_pk_index_open_login (NULL);
+		if (!pv->index)
+			pv->index = gkr_pk_index_open_session (NULL);
 	}
 	
 	return pv->index;
@@ -661,7 +657,7 @@ gkr_pk_storage_get_store_password (GkrPkStorage *storage, GQuark location, gkrco
 	gchar *custom_label = NULL;
 	gchar *secondary;
 	gchar *display = NULL;
-	gboolean ret;
+	gboolean ret = TRUE;
 	guint flags;
 	GkrPkIndex *index;
 	GkrKeyring *login;
@@ -670,7 +666,6 @@ gkr_pk_storage_get_store_password (GkrPkStorage *storage, GQuark location, gkrco
 	g_return_val_if_fail (result != NULL, FALSE);
 
 	index = gkr_pk_storage_index (storage, location);
-	g_return_val_if_fail (index, FALSE);
 
 	/*
 	 * We save the password while still here in this function.
@@ -680,7 +675,7 @@ gkr_pk_storage_get_store_password (GkrPkStorage *storage, GQuark location, gkrco
 	 */
 
 	/* See if we can just use the login keyring password for this */
-	if (gkr_keyring_login_is_usable ()) {
+	if (index && gkr_keyring_login_is_usable () && gkr_pk_index_allows_secrets (index)) {
 		login = gkr_keyrings_get_login ();
 		g_return_val_if_fail (login, FALSE);
 		g_return_val_if_fail (login->password, FALSE);
@@ -710,7 +705,7 @@ gkr_pk_storage_get_store_password (GkrPkStorage *storage, GQuark location, gkrco
 
 	gkr_ask_request_set_location (ask, location);
 			
-	if (gkr_keyring_login_is_usable ())
+	if (index && gkr_pk_index_allows_secrets (index))
 		gkr_ask_request_set_check_option (ask, prepare_ask_check (type));
 		
 	/* Prompt the user */
@@ -745,7 +740,8 @@ gkr_pk_storage_get_load_password (GkrPkStorage *storage, GQuark location, gkrcon
 	GkrAskRequest *ask;
 	gchar *stype, *secondary;
 	gchar *display = NULL;
-	gboolean ret;
+	const gchar *password;
+	gboolean ret = TRUE;
 	GkrPkIndex *index;
 	gint st;
 	guint flags;
@@ -781,7 +777,6 @@ gkr_pk_storage_get_load_password (GkrPkStorage *storage, GQuark location, gkrcon
 	}
 	
 	index = gkr_pk_storage_index (storage, location);
-	g_return_val_if_fail (index, FALSE);
 	
 	/*
 	 * The password prompting is somewhat convoluted with the end goal of 
@@ -798,7 +793,20 @@ gkr_pk_storage_get_load_password (GkrPkStorage *storage, GQuark location, gkrcon
 	if (st == 2) {
 		*result = gkr_pk_index_get_secret (index, digest);
 		if (*result != NULL)
-			 return TRUE;
+			return TRUE;
+		
+		/* 
+		 * COMPATIBILITY: This is for compatibility with old versions 2.22, which 
+		 * stored a location/filename based password in the login keyring.
+		 * This is wrong for locations with  more than one password, so we 
+		 * migrate transparently to the new style (above).
+		 */ 
+		password = gkr_keyring_login_lookup_secret (GNOME_KEYRING_ITEM_ENCRYPTION_KEY_PASSWORD,
+				                            "pk-object", gkr_location_to_string (location), NULL);
+		if (password != NULL) {
+			*result = gkr_secure_strdup (password);
+			return TRUE;
+		}
 		
 	/* If we've already tried this password unsuccesfully, then clear */
 	} else {
@@ -830,7 +838,7 @@ gkr_pk_storage_get_load_password (GkrPkStorage *storage, GQuark location, gkrcon
 
 	gkr_ask_request_set_location (ask, location);
 			
-	if (gkr_keyring_login_is_usable ())
+	if (index && gkr_pk_index_allows_secrets (index))
 		gkr_ask_request_set_check_option (ask, prepare_ask_check (type));
 		
 	/* Prompt the user */
